unit OL_SelectionChange_IMPL;

interface

uses
  ComObj, ComServ, ActiveX, Variants, adxAddIn, OL_SelectionChange_TLB, StdVcl,
  Outlook2000, Classes, Dialogs, SysUtils;

type
  TOL_Addin = class(TadxAddin, IOL_Addin)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    procedure adxCOMAddInModuleAddInInitialize(Sender: TObject);
    procedure adxCOMAddInModuleAddInFinalize(Sender: TObject);
  private
    FExplorers: TExplorers;
    FExplList: TList;
    FInspectors: TInspectors;
    FInspList: TList;
    FLastEntryID: WideString;
    {$IFDEF VER140}
    procedure DoNewExplorer(ASender: TObject; var Explorer: OleVariant);
    procedure DoNewInspector(ASender: TObject; var Inspector: OleVariant);
    {$ELSE}
    procedure DoNewExplorer(ASender: TObject; const Explorer: _Explorer);
    procedure DoNewInspector(ASender: TObject; const Inspector: _Inspector);
    {$ENDIF}
    procedure DoCloseExplorer(Sender: TObject);
    procedure DoCloseInspector(Sender: TObject);
    procedure DoSelectionChange(Sender: TObject);
    procedure MyActionOnSelectionChange(Item: OLEVAriant);
  end;

implementation

{$R *.dfm}

{ TAddInModule }

{$IFDEF VER140}
procedure TAddInModule.adxCOMAddInModuleAddInInitialize(Sender: TObject);
var
  activeExplorer: OleVariant;
  activeInspector: OleVariant;
begin
  FExplList := TList.Create;
  FExplorers := TExplorers.Create(nil);
  FExplorers.ConnectTo(OutlookApp.Explorers);
  FExplorers.OnNewExplorer := DoNewExplorer;
  if OutlookApp.Explorers.Count > 0 then begin
    activeExplorer:= OutlookApp.ActiveExplorer;
    DoNewExplorer(nil, activeExplorer);
    activeExplorer:= NULL;
  end;
  FInspList := TList.Create;
  FInspectors := TInspectors.Create(nil);
  FInspectors.ConnectTo(OutlookApp.Inspectors);
  FInspectors.OnNewInspector := DoNewInspector;
  if OutlookApp.Inspectors.Count > 0 then begin
    activeInspector:= OutlookApp.ActiveInspector;
    DoNewInspector(nil, activeInspector);
    activeInspector:= NULL;
  end;
end;

{$ELSE}
procedure TAddInModule.adxCOMAddInModuleAddInInitialize(Sender: TObject);
begin
  FExplList := TList.Create;
  FExplorers := TExplorers.Create(nil);
  FExplorers.ConnectTo(OutlookApp.Explorers);
  FExplorers.OnNewExplorer := DoNewExplorer;
  if OutlookApp.Explorers.Count > 0 then
    DoNewExplorer(nil, OutlookApp.ActiveExplorer);
  FInspList := TList.Create;
  FInspectors := TInspectors.Create(nil);
  FInspectors.ConnectTo(OutlookApp.Inspectors);
  FInspectors.OnNewInspector := DoNewInspector;
  if OutlookApp.Inspectors.Count > 0 then
    DoNewInspector(nil, OutlookApp.ActiveInspector);
end;
{$ENDIF}

procedure TAddInModule.adxCOMAddInModuleAddInFinalize(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to FExplList.Count - 1 do begin
    TExplorer(FExplList.Items[i]).Free;
    FExplList.Delete(i);
  end;
  FreeAndNil(FExplList);
  FreeAndNil(FExplorers);
  for i := 0 to FInspList.Count - 1 do begin
    TInspector(FInspList.Items[i]).Free;
    FInspList.Delete(i);
  end;
  FreeAndNil(FInspList);
  FreeAndNil(FInspectors);
end;

{$IFDEF VER140}
procedure TAddInModule.DoNewExplorer(ASender: TObject; var Explorer: OleVariant);
var
  Expl: TExplorer;
begin
  Expl := TExplorer.Create(nil);
  Expl.ConnectTo(_Explorer(TVarData(Explorer).VDispatch));
  Expl.OnSelectionChange := DoSelectionChange;
  Expl.OnClose := DoCloseExplorer;
  FExplList.Add(Expl);
end;

{$ELSE}
procedure TAddInModule.DoNewExplorer(ASender: TObject; const Explorer: _Explorer);
var
  Expl: TExplorer;
begin
  Expl := TExplorer.Create(nil);
  Expl.ConnectTo(Explorer);
  Expl.OnSelectionChange := DoSelectionChange;
  Expl.OnClose := DoCloseExplorer;
  FExplList.Add(Expl);
end;
{$ENDIF}

{$IFDEF VER140}
procedure TAddInModule.DoNewInspector(ASender: TObject; var Inspector: OleVariant);
var
  Insp: TInspector;
begin
  Insp := TInspector.Create(nil);
  Insp.ConnectTo(_Inspector(TVarData(Inspector).VDispatch));
  Insp.OnClose := DoSelectionChange;
  Insp.OnClose := DoCloseInspector;
  FInspList.Add(Insp);
  DoSelectionChange(nil);
end;

{$ELSE}
procedure TAddInModule.DoNewInspector(ASender: TObject; const Inspector: _Inspector);
var
  Insp: TInspector;
begin
  Insp := TInspector.Create(nil);
  Insp.ConnectTo(Inspector);
  Insp.OnClose := DoSelectionChange;
  Insp.OnClose := DoCloseInspector;
  FInspList.Add(Insp);
  DoSelectionChange(nil);
end;
{$ENDIF}

procedure TAddInModule.DoCloseExplorer(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to FExplList.Count - 1 do
    if Sender = FExplList.Items[i] then begin
      TExplorer(FExplList.Items[i]).Free;
      FExplList.Delete(i);
      Break;
    end;
end;

procedure TAddInModule.DoCloseInspector(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to FInspList.Count - 1 do
    if Sender = FInspList.Items[i] then begin
      TInspector(FInspList.Items[i]).Free;
      FInspList.Delete(i);
      Break;
    end;
end;

procedure TAddInModule.DoSelectionChange(Sender: TObject);
var
  IInsp: _Inspector;
  VIntf: OLEVariant;
begin
  IInsp := OutlookApp.ActiveInspector;
  VIntf := NULL;
  try
    if Assigned(IInsp) then
      VIntf := IInsp.CurrentItem
    else begin
      VIntf := OutlookApp.ActiveExplorer.Selection;
      if VIntf.Count > 0 then
        VIntf := VIntf.Item(1)
      else
        VIntf := NULL;
    end;
    if not VarIsNULL(VIntf) then
      if VIntf.Class = olMail then
        MyActionOnSelectionChange(VIntf)
  finally
    VIntf := UnAssigned;
    IInsp := nil;
  end;
end;

procedure TAddInModule.MyActionOnSelectionChange(Item: OLEVAriant);
begin
  if FLastEntryID <> Item.EntryID then begin
    ShowMessage(Item.Subject);
    FLastEntryID := Item.EntryID;
  end;
end;

initialization
  TadxFactory.Create(ComServer, TOL_Addin, CLASS_OL_Addin, TAddInModule);

end.
